@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2002-2004 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,8 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
+ *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 
@@ -27,10 +24,7 @@
 #include <sys/types.h>
 #include <iconv.h>
 
-#ifdef __cplusplus
-extern "C" {
-#pragma }
-#endif /* __cplusplus */
+G_BEGIN_DECLS
 
 char *g_mime_iconv_strdup (iconv_t cd, const char *string);
 char *g_mime_iconv_strndup (iconv_t cd, const char *string, size_t n);
@@ -41,8 +35,6 @@ char *g_mime_iconv_locale_to_utf8_length (const char *string, size_t n);
 char *g_mime_iconv_utf8_to_locale (const char *string);
 char *g_mime_iconv_utf8_to_locale_length (const char *string, size_t n);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 #endif /* __GMIME_ICONV_UTILS_H__ */
