@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2002 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,8 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
+ *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 
@@ -107,7 +104,7 @@ g_mime_message_part_finalize (GObject *object)
 	GMimeMessagePart *part = (GMimeMessagePart *) object;
 	
 	if (part->message)
-		g_mime_object_unref (GMIME_OBJECT (part->message));
+		g_object_unref (part->message);
 	
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -126,7 +123,7 @@ message_part_add_header (GMimeObject *object, const char *header, const char *va
 	/* Make sure that the header is a Content-* header, else it
            doesn't belong on a message part */
 	
-	if (!strncasecmp ("Content-", header, 8))
+	if (!g_ascii_strncasecmp ("Content-", header, 8))
 		GMIME_OBJECT_CLASS (parent_class)->add_header (object, header, value);
 }
 
@@ -134,13 +131,13 @@ static void
 message_part_set_header (GMimeObject *object, const char *header, const char *value)
 {
 	/* RFC 1864 states that you cannot set a Content-MD5 on a message part */
-	if (!strcasecmp ("Content-MD5", header))
+	if (!g_ascii_strcasecmp ("Content-MD5", header))
 		return;
 	
 	/* Make sure that the header is a Content-* header, else it
            doesn't belong on a message part */
 	
-	if (!strncasecmp ("Content-", header, 8))
+	if (!g_ascii_strncasecmp ("Content-", header, 8))
 		GMIME_OBJECT_CLASS (parent_class)->set_header (object, header, value);
 }
 
@@ -150,7 +147,7 @@ message_part_get_header (GMimeObject *object, const char *header)
 	/* Make sure that the header is a Content-* header, else it
            doesn't belong on a message part */
 	
-	if (!strncasecmp ("Content-", header, 8))
+	if (!g_ascii_strncasecmp ("Content-", header, 8))
 		return GMIME_OBJECT_CLASS (parent_class)->get_header (object, header);
 	else
 		return NULL;
@@ -162,7 +159,7 @@ message_part_remove_header (GMimeObject *object, const char *header)
 	/* Make sure that the header is a Content-* header, else it
            doesn't belong on a message part */
 	
-	if (!strncasecmp ("Content-", header, 8))
+	if (!g_ascii_strncasecmp ("Content-", header, 8))
 		GMIME_OBJECT_CLASS (parent_class)->remove_header (object, header);
 }
 
@@ -186,23 +183,20 @@ message_part_write_to_stream (GMimeObject *object, GMimeStream *stream)
 	ssize_t nwritten, total = 0;
 	
 	/* write the content headers */
-	nwritten = g_mime_header_write_to_stream (object->headers, stream);
-	if (nwritten == -1)
+	if ((nwritten = g_mime_header_write_to_stream (object->headers, stream)) == -1)
 		return -1;
 	
 	total += nwritten;
 	
 	/* terminate the headers */
-	nwritten = g_mime_stream_write (stream, "\n", 1);
-	if (nwritten == -1)
+	if ((nwritten = g_mime_stream_write (stream, "\n", 1)) == -1)
 		return -1;
 	
 	total += nwritten;
 	
 	/* write the message */
 	if (part->message) {
-		nwritten = g_mime_object_write_to_stream (GMIME_OBJECT (part->message), stream);
-		if (nwritten == -1)
+		if ((nwritten = g_mime_object_write_to_stream (GMIME_OBJECT (part->message), stream)) == -1)
 			return -1;
 		
 		total += nwritten;
@@ -255,7 +249,7 @@ g_mime_message_part_new_with_message (const char *subtype, GMimeMessage *message
 	
 	part = g_mime_message_part_new (subtype);
 	part->message = message;
-	g_mime_object_ref (GMIME_OBJECT (message));
+	g_object_ref (message);
 	
 	return part;
 }
@@ -274,10 +268,10 @@ g_mime_message_part_set_message (GMimeMessagePart *part, GMimeMessage *message)
 	g_return_if_fail (GMIME_IS_MESSAGE_PART (part));
 	
 	if (message)
-		g_mime_object_ref (GMIME_OBJECT (message));
+		g_object_ref (message);
 	
 	if (part->message)
-		g_mime_object_unref (GMIME_OBJECT (part->message));
+		g_object_unref (part->message);
 	
 	part->message = message;
 }
