@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2002-2004 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,8 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
+ *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 
@@ -118,13 +115,13 @@ static void
 message_partial_add_header (GMimeObject *object, const char *header, const char *value)
 {
 	/* RFC 1864 states that you cannot set a Content-MD5 on a message part */
-	if (!strcasecmp ("Content-MD5", header))
+	if (!g_ascii_strcasecmp ("Content-MD5", header))
 		return;
 	
 	/* Make sure that the header is a Content-* header, else it
            doesn't belong on a mime part */
 	
-	if (!strncasecmp ("Content-", header, 8))
+	if (!g_ascii_strncasecmp ("Content-", header, 8))
 		GMIME_OBJECT_CLASS (parent_class)->add_header (object, header, value);
 }
 
@@ -132,13 +129,13 @@ static void
 message_partial_set_header (GMimeObject *object, const char *header, const char *value)
 {
 	/* RFC 1864 states that you cannot set a Content-MD5 on a message part */
-	if (!strcasecmp ("Content-MD5", header))
+	if (!g_ascii_strcasecmp ("Content-MD5", header))
 		return;
 	
 	/* Make sure that the header is a Content-* header, else it
            doesn't belong on a mime part */
 	
-	if (!strncasecmp ("Content-", header, 8))
+	if (!g_ascii_strncasecmp ("Content-", header, 8))
 		GMIME_OBJECT_CLASS (parent_class)->set_header (object, header, value);
 }
 
@@ -153,10 +150,10 @@ message_partial_set_content_type (GMimeObject *object, GMimeContentType *content
 	partial->id = g_strdup (value);
 	
 	value = g_mime_content_type_get_parameter (content_type, "number");
-	partial->number = value ? atoi (value) : -1;
+	partial->number = value ? strtol (value, NULL, 10) : -1;
 	
 	value = g_mime_content_type_get_parameter (content_type, "total");
-	partial->total = value ? atoi (value) : -1;
+	partial->total = value ? strtol (value, NULL, 10) : -1;
 	
 	GMIME_OBJECT_CLASS (parent_class)->set_content_type (object, content_type);
 }
@@ -225,7 +222,7 @@ g_mime_message_partial_get_id (GMimeMessagePartial *partial)
  *
  * Gets the message/partial part number.
  *
- * Returns the message/partial part number or -1 on fail.
+ * Returns the message/partial part number or %-1 on fail.
  **/
 int
 g_mime_message_partial_get_number (GMimeMessagePartial *partial)
@@ -293,13 +290,11 @@ g_mime_message_partial_reconstruct_message (GMimeMessagePartial **partials, size
 	
 	g_return_val_if_fail (num > 0, NULL);
 	
-	id = g_mime_message_partial_get_id (partials[0]);
-	if (!id)
+	if (!(id = g_mime_message_partial_get_id (partials[0])))
 		return NULL;
 	
 	/* get them into the correct order... */
-	qsort ((void *) partials, num, sizeof (gpointer),
-	       partial_compare);
+	qsort ((void *) partials, num, sizeof (gpointer), partial_compare);
 	
 	/* only the last message/partial part is REQUIRED to have the total parameter */
 	total = g_mime_message_partial_get_total (partials[num - 1]);
@@ -354,7 +349,8 @@ header_copy (const char *name, const char *value, gpointer user_data)
 {
 	GMimeMessage *message = (GMimeMessage *) user_data;
 	
-	g_mime_object_add_header (GMIME_OBJECT (message), name, value);
+	if (value)
+		g_mime_object_add_header (GMIME_OBJECT (message), name, value);
 }
 
 static GMimeMessage *
@@ -390,10 +386,11 @@ g_mime_message_partial_split_message (GMimeMessage *message, size_t max_size, si
 	GMimeMessagePartial *partial;
 	GMimeStream *stream, *substream;
 	GMimeDataWrapper *wrapper;
+	const unsigned char *buf;
 	GPtrArray *parts;
+	size_t len, end;
 	const char *id;
 	off_t start;
-	size_t len;
 	int i;
 	
 	*nparts = 0;
@@ -401,7 +398,6 @@ g_mime_message_partial_split_message (GMimeMessage *message, size_t max_size, si
 	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
 	
 	stream = g_mime_stream_mem_new ();
-	
 	if (g_mime_object_write_to_stream (GMIME_OBJECT (message), stream) == -1) {
 		g_object_unref (stream);
 		return NULL;
@@ -416,18 +412,34 @@ g_mime_message_partial_split_message (GMimeMessage *message, size_t max_size, si
 		g_object_unref (stream);
 		g_object_ref (message);
 		
-		messages = g_malloc (sizeof (void *) * 2);
+		messages = g_malloc (sizeof (void *));
 		messages[0] = message;
 		*nparts = 1;
 		
 		return messages;
 	}
 	
+	start = 0;
 	parts = g_ptr_array_new ();
-	for (i = 0, start = 0; start < len; i++) {
-		substream = g_mime_stream_substream (stream, start, MIN (len, start + max_size));
+	buf = ((GMimeStreamMem *) stream)->buffer->data;
+	
+	while (start < len) {
+		/* Preferably, we'd split on whole-lines if we can,
+		 * but if that's not possible, split on max size */
+		if ((end = MIN (len, start + max_size)) < len) {
+			register size_t ebx; /* end boundary */
+			
+			ebx = end;
+			while (ebx > (start + 1) && ebx[buf] != '\n')
+				ebx--;
+			
+			if (ebx[buf] == '\n')
+				end = ebx + 1;
+		}
+		
+		substream = g_mime_stream_substream (stream, start, end);
 		g_ptr_array_add (parts, substream);
-		start += max_size;
+		start = end;
 	}
 	
 	id = g_mime_message_get_message_id (message);
