@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2000-2004 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,8 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
+ *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 
@@ -50,10 +47,7 @@ static void message_remove_header (GMimeObject *object, const char *header);
 static char *message_get_headers (GMimeObject *object);
 static ssize_t message_write_to_stream (GMimeObject *object, GMimeStream *stream);
 
-static void message_set_sender (GMimeMessage *message, const char *sender);
-static void message_set_reply_to (GMimeMessage *message, const char *reply_to);
 static void message_add_recipients_from_string (GMimeMessage *message, char *type, const char *string);
-static void message_set_subject (GMimeMessage *message, const char *subject);
 
 static ssize_t write_addrspec (GMimeStream *stream, const char *name, const char *value);
 static ssize_t write_received (GMimeStream *stream, const char *name, const char *value);
@@ -420,10 +414,11 @@ static ssize_t
 write_received (GMimeStream *stream, const char *name, const char *value)
 {
 	struct _received_part *parts, *part, *tail;
-	const char *lwsp, *inptr;
+	const char *inptr, *lwsp = NULL;
 	ssize_t nwritten;
 	GString *str;
-	int len, i;
+	size_t len;
+	guint i;
 	
 	while (is_lwsp (*value))
 		value++;
@@ -443,6 +438,7 @@ write_received (GMimeStream *stream, const char *name, const char *value)
 		for (i = 0; i < G_N_ELEMENTS (received_tokens); i++) {
 			if (!strncmp (inptr, received_tokens[i].token, received_tokens[i].len)) {
 				if (inptr > part->start) {
+					g_assert (lwsp != NULL);
 					part->len = lwsp - part->start;
 					
 					part = g_alloca (sizeof (struct _received_part));
@@ -596,20 +592,27 @@ static gboolean
 process_header (GMimeObject *object, const char *header, const char *value)
 {
 	GMimeMessage *message = (GMimeMessage *) object;	
+	InternetAddressList *addrlist;
 	int offset, i;
 	time_t date;
 	
 	for (i = 0; headers[i]; i++) {
-		if (!strcasecmp (headers[i], header))
+		if (!g_ascii_strcasecmp (headers[i], header))
 			break;
 	}
 	
 	switch (i) {
 	case HEADER_FROM:
-		message_set_sender (message, value);
+		g_free (message->from);
+		addrlist = internet_address_parse_string (value);
+		message->from = internet_address_list_to_string (addrlist, FALSE);
+		internet_address_list_destroy (addrlist);
 		break;
 	case HEADER_REPLY_TO:
-		message_set_reply_to (message, value);
+		g_free (message->reply_to);
+		addrlist = internet_address_parse_string (value);
+		message->reply_to = internet_address_list_to_string (addrlist, FALSE);
+		internet_address_list_destroy (addrlist);
 		break;
 	case HEADER_TO:
 		message_add_recipients_from_string (message, GMIME_RECIPIENT_TYPE_TO, value);
@@ -621,7 +624,8 @@ process_header (GMimeObject *object, const char *header, const char *value)
 		message_add_recipients_from_string (message, GMIME_RECIPIENT_TYPE_BCC, value);
 		break;
 	case HEADER_SUBJECT:
-		message_set_subject (message, value);
+		g_free (message->subject);
+		message->subject = g_mime_utils_header_decode_text (value);
 		break;
 	case HEADER_DATE:
 		if (value) {
@@ -645,13 +649,13 @@ process_header (GMimeObject *object, const char *header, const char *value)
 static void
 message_add_header (GMimeObject *object, const char *header, const char *value)
 {
-	if (!strcasecmp ("MIME-Version", header))
+	if (!g_ascii_strcasecmp ("MIME-Version", header))
 		return;
 	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
 	
-	if (strncasecmp ("Content-", header, 8)) {
+	if (g_ascii_strncasecmp ("Content-", header, 8)) {
 		if (process_header (object, header, value))
 			g_mime_header_add (object->headers, header, value);
 		else
@@ -665,13 +669,13 @@ message_add_header (GMimeObject *object, const char *header, const char *value)
 static void
 message_set_header (GMimeObject *object, const char *header, const char *value)
 {
-	if (!strcasecmp ("MIME-Version", header))
+	if (!g_ascii_strcasecmp ("MIME-Version", header))
 		return;
 	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
 	
-	if (strncasecmp ("Content-", header, 8)) {
+	if (g_ascii_strncasecmp ("Content-", header, 8)) {
 		if (process_header (object, header, value))
 			g_mime_header_set (object->headers, header, value);
 		else
@@ -688,10 +692,10 @@ message_get_header (GMimeObject *object, const char *header)
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
 	
-	if (!strcasecmp ("MIME-Version", header))
+	if (!g_ascii_strcasecmp ("MIME-Version", header))
 		return "1.0";
 	
-	if (strncasecmp ("Content-", header, 8) != 0)
+	if (g_ascii_strncasecmp ("Content-", header, 8) != 0)
 		return GMIME_OBJECT_CLASS (parent_class)->get_header (object, header);
 	else if (((GMimeMessage *) object)->mime_part)
 		return g_mime_object_get_header (((GMimeMessage *) object)->mime_part, header);
@@ -706,17 +710,17 @@ message_remove_header (GMimeObject *object, const char *header)
 	InternetAddressList *addrlist;
 	int i;
 	
-	if (!strcasecmp ("MIME-Version", header))
+	if (!g_ascii_strcasecmp ("MIME-Version", header))
 		return;
 	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
 	
-	if (!strncasecmp ("Content-", header, 8))
+	if (!g_ascii_strncasecmp ("Content-", header, 8))
 		return;
 	
 	for (i = 0; headers[i]; i++) {
-		if (!strcasecmp (headers[i], header))
+		if (!g_ascii_strcasecmp (headers[i], header))
 			break;
 	}
 	
@@ -791,8 +795,8 @@ message_get_headers (GMimeObject *object)
 	}
 	
 	g_object_unref (stream);
-	g_byte_array_append (ba, "", 1);
-	str = ba->data;
+	g_byte_array_append (ba, (unsigned char *) "", 1);
+	str = (char *) ba->data;
 	g_byte_array_free (ba, FALSE);
 	
 	return str;
@@ -861,16 +865,6 @@ g_mime_message_new (gboolean pretty_headers)
 }
 
 
-static void
-message_set_sender (GMimeMessage *message, const char *sender)
-{
-	if (message->from)
-		g_free (message->from);
-	
-	message->from = g_strstrip (g_strdup (sender));
-}
-
-
 /**
  * g_mime_message_set_sender:
  * @message: MIME Message to change
@@ -888,9 +882,10 @@ g_mime_message_set_sender (GMimeMessage *message, const char *sender)
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	g_return_if_fail (sender != NULL);
 	
-	message_set_sender (message, sender);
+	g_free (message->from);
 	
 	addrlist = internet_address_parse_string (sender);
+	message->from = internet_address_list_to_string (addrlist, FALSE);
 	encoded = internet_address_list_to_string (addrlist, TRUE);
 	internet_address_list_destroy (addrlist);
 	
@@ -916,16 +911,6 @@ g_mime_message_get_sender (GMimeMessage *message)
 }
 
 
-static void
-message_set_reply_to (GMimeMessage *message, const char *reply_to)
-{
-	if (message->reply_to)
-		g_free (message->reply_to);
-	
-	message->reply_to = g_strstrip (g_strdup (reply_to));
-}
-
-
 /**
  * g_mime_message_set_reply_to:
  * @message: MIME Message to change
@@ -939,7 +924,9 @@ g_mime_message_set_reply_to (GMimeMessage *message, const char *reply_to)
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	g_return_if_fail (reply_to != NULL);
 	
-	message_set_reply_to (message, reply_to);
+	g_free (message->reply_to);
+	message->reply_to = g_strstrip (g_strdup (reply_to));
+	
 	g_mime_header_set (GMIME_OBJECT (message)->headers, "Reply-To", message->reply_to);
 }
 
@@ -965,12 +952,10 @@ static void
 sync_recipient_header (GMimeMessage *message, const char *type)
 {
 	const InternetAddressList *recipients;
+	char *string;
 	
 	/* sync the specified recipient header */
-	recipients = g_mime_message_get_recipients (message, type);
-	if (recipients) {
-		char *string;
-		
+	if ((recipients = g_mime_message_get_recipients (message, type))) {
 		string = internet_address_list_to_string (recipients, TRUE);
 		g_mime_header_set (GMIME_OBJECT (message)->headers, type, string);
 		g_free (string);
@@ -1022,8 +1007,7 @@ message_add_recipients_from_string (GMimeMessage *message, char *type, const cha
 	recipients = g_hash_table_lookup (message->recipients, type);
 	g_hash_table_remove (message->recipients, type);
 	
-	addrlist = internet_address_parse_string (string);
-	if (addrlist) {
+	if ((addrlist = internet_address_parse_string (string))) {
 		recipients = internet_address_list_concat (recipients, addrlist);
 		internet_address_list_destroy (addrlist);
 	}
@@ -1079,13 +1063,47 @@ g_mime_message_get_recipients (GMimeMessage *message, const char *type)
 }
 
 
-static void
-message_set_subject (GMimeMessage *message, const char *subject)
+static const char *recipient_types[] = {
+	GMIME_RECIPIENT_TYPE_TO,
+	GMIME_RECIPIENT_TYPE_CC,
+	GMIME_RECIPIENT_TYPE_BCC
+};
+
+/**
+ * g_mime_message_get_all_recipients:
+ * @message: MIME Message
+ *
+ * Gets the complete list of recipients for @message.
+ *
+ * Returns a newly allocated #InternetAddressList containing all
+ * recipients of the message.
+ **/
+InternetAddressList *
+g_mime_message_get_all_recipients (GMimeMessage *message)
 {
-	if (message->subject)
-		g_free (message->subject);
+	InternetAddressList *list, *tail, *node, *recipients;
+	guint i;
 	
-	message->subject = g_strstrip (g_strdup (subject));
+	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
+	
+	list = NULL;
+	tail = (InternetAddressList *) &list;
+	
+	for (i = 0; i < G_N_ELEMENTS (recipient_types); i++) {
+		recipients = g_hash_table_lookup (message->recipients, recipient_types[i]);
+		while (recipients != NULL) {
+			internet_address_ref (recipients->address);
+			node = g_new (InternetAddressList, 1);
+			node->address = recipients->address;
+			node->next = NULL;
+			tail->next = node;
+			tail = node;
+			
+			recipients = recipients->next;
+		}
+	}
+	
+	return list;
 }
 
 
@@ -1094,7 +1112,7 @@ message_set_subject (GMimeMessage *message, const char *subject)
  * @message: MIME Message
  * @subject: Subject string
  *
- * Set the Subject field on a MIME Message.
+ * Set the unencoded UTF-8 Subject field on a MIME Message.
  **/
 void
 g_mime_message_set_subject (GMimeMessage *message, const char *subject)
@@ -1104,7 +1122,8 @@ g_mime_message_set_subject (GMimeMessage *message, const char *subject)
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	g_return_if_fail (subject != NULL);
 	
-	message_set_subject (message, subject);
+	g_free (message->subject);
+	message->subject = g_strstrip (g_strdup (subject));
 	
 	encoded = g_mime_utils_header_encode_text (message->subject);
 	g_mime_header_set (GMIME_OBJECT (message)->headers, "Subject", encoded);
@@ -1118,7 +1137,7 @@ g_mime_message_set_subject (GMimeMessage *message, const char *subject)
  *
  * Gets the message's subject.
  *
- * Returns the Subject field on a MIME Message.
+ * Returns the unencoded UTF-8 Subject field on a MIME Message.
  **/
 const char *
 g_mime_message_get_subject (GMimeMessage *message)
@@ -1212,9 +1231,7 @@ g_mime_message_set_message_id (GMimeMessage *message, const char *message_id)
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	g_return_if_fail (message_id != NULL);
 	
-	if (message->message_id)
-		g_free (message->message_id);
-	
+	g_free (message->message_id);
 	message->message_id = g_strstrip (g_strdup (message_id));
 	
 	msgid = g_strdup_printf ("<%s>", message_id);
@@ -1354,7 +1371,7 @@ g_mime_message_set_mime_part (GMimeMessage *message, GMimeObject *mime_part)
  * WARNING: This interface is deprecated. Use
  * g_mime_object_write_to_stream() instead.
  *
- * Returns -1 on fail.
+ * Returns %-1 on fail.
  **/
 ssize_t
 g_mime_message_write_to_stream (GMimeMessage *message, GMimeStream *stream)
@@ -1411,8 +1428,8 @@ handle_multipart_alternative (GMimeMultipart *multipart, gboolean want_plain, gb
 		
 		type = g_mime_object_get_content_type (mime_part);
 		if (g_mime_content_type_is_type (type, "text", "*")) {
-			if (!text_part || !strcasecmp (type->subtype, want_plain ? "plain" : "html")) {
-				*is_html = !strcasecmp (type->subtype, "html");
+			if (!text_part || !g_ascii_strcasecmp (type->subtype, want_plain ? "plain" : "html")) {
+				*is_html = !g_ascii_strcasecmp (type->subtype, "html");
 				text_part = mime_part;
 			}
 		}
@@ -1447,18 +1464,17 @@ handle_multipart_mixed (GMimeMultipart *multipart, gboolean want_plain, gboolean
 					text_part = mime_part;
 			}
 		} else if (g_mime_content_type_is_type (type, "text", "*")) {
-			if (!strcasecmp (type->subtype, want_plain ? "plain" : "html")) {
+			if (!g_ascii_strcasecmp (type->subtype, want_plain ? "plain" : "html")) {
 				/* we got what we came for */
-				*is_html = !strcasecmp (type->subtype, "html");
+				*is_html = !g_ascii_strcasecmp (type->subtype, "html");
 				return mime_part;
 			}
 			
-			/* if we haven't yet found a text part or if
-                           it is a type we can understand and it is
-                           the first of that type, save it */
-			if (!text_part || (!strcasecmp (type->subtype, "plain") && (first_type &&
-					   strcasecmp (type->subtype, first_type->subtype) != 0))) {
-				*is_html = !strcasecmp (type->subtype, "html");
+			/* if we haven't yet found a text part or if it is a type we can
+			 * understand and it is the first of that type, save it */
+			if (!text_part || (!g_ascii_strcasecmp (type->subtype, "plain") && (first_type &&
+					   g_ascii_strcasecmp (type->subtype, first_type->subtype) != 0))) {
+				*is_html = !g_ascii_strcasecmp (type->subtype, "html");
 				text_part = mime_part;
 				first_type = type;
 			}
@@ -1480,6 +1496,8 @@ handle_multipart_mixed (GMimeMultipart *multipart, gboolean want_plain, gboolean
  * Attempts to get the body of the message in the preferred format
  * specified by @want_plain.
  *
+ * WARNING: This interface is deprecated.
+ *
  * Returns the prefered form of the message body. Sets the value of
  * @is_html to %TRUE if the part returned is in HTML format, otherwise
  * %FALSE.
