@@ -23,6 +23,7 @@
 #endif
 
 #include <string.h>
+#include <limits.h>
 #include <ctype.h>
 #include <errno.h>
 
@@ -103,20 +104,31 @@ decode_lwsp (const char **in)
 	*in = inptr;
 }
 
+#define INT_OVERFLOW(x,d) (((x) > (INT_MAX / 10)) || ((x) == (INT_MAX / 10) && (d) > (INT_MAX % 10)))
+
 static int
 decode_int (const char **in)
 {
-	const char *inptr = *in;
-	int n = 0;
+	const unsigned char *inptr;
+	int digit, n = 0;
 	
-	decode_lwsp (&inptr);
+	decode_lwsp (in);
 	
+	inptr = (const unsigned char *) *in;
 	while (isdigit ((int) *inptr)) {
-		n = n * 10 + (*inptr - '0');
+		digit = (*inptr - '0');
+		if (INT_OVERFLOW (n, digit)) {
+			while (isdigit ((int) *inptr))
+				inptr++;
+			break;
+		}
+		
+		n = (n * 10) + digit;
+		
 		inptr++;
 	}
 	
-	*in = inptr;
+	*in = (const char *) inptr;
 	
 	return n;
 }
@@ -286,10 +298,10 @@ decode_param (const char **in, char **paramp, char **valuep, int *id, gboolean *
 		value = decode_value (&inptr);
 		
 		if (!is_rfc2184 && value) {
-			if (!strncmp (value, "=?", 2)) {
-				/* We have a broken param value that is rfc2047 encoded.
-				 * Since both Outlook and Netscape/Mozilla do this, we
-				 * should handle this case.
+			if (strstr (value, "=?") != NULL) {
+				/* We (may) have a broken param value that is rfc2047
+				 * encoded. Since both Outlook and Netscape/Mozilla do
+				 * this, we should handle this case.
 				 */
 				
 				if ((val = g_mime_utils_header_decode_text (value))) {
@@ -352,16 +364,11 @@ rfc2184_sort_cb (const void *v0, const void *v1)
 #define HEXVAL(c) (isdigit (c) ? (c) - '0' : tolower (c) - 'a' + 10)
 
 static size_t
-hex_decode (const unsigned char *in, size_t len, unsigned char *out)
+hex_decode (const char *in, size_t len, char *out)
 {
-	register const unsigned char *inptr;
-	register unsigned char *outptr;
-	const unsigned char *inend;
-	
-	inptr = in;
-	inend = in + len;
-	
-	outptr = out;
+	register const unsigned char *inptr = (const unsigned char *) in;
+	register unsigned char *outptr = (unsigned char *) out;
+	const unsigned char *inend = inptr + len;
 	
 	while (inptr < inend) {
 		if (*inptr == '%') {
@@ -376,7 +383,7 @@ hex_decode (const unsigned char *in, size_t len, unsigned char *out)
 	
 	*outptr = '\0';
 	
-	return outptr - out;
+	return ((char *) outptr) - out;
 }
 
 static const char *
@@ -623,7 +630,7 @@ decode_param_list (const char *in)
  *
  * Creates a parameter list based on the input string.
  *
- * Returns a GMimeParam structure based on @string.
+ * Returns a #GMimeParam structure based on @string.
  **/
 GMimeParam *
 g_mime_param_new_from_string (const char *string)
@@ -719,21 +726,24 @@ g_mime_param_append_param (GMimeParam *params, GMimeParam *param)
 
 /* FIXME: I wrote this in a quick & dirty fasion - it may not be 100% correct */
 static char *
-encode_param (const unsigned char *in, gboolean *encoded)
+encode_param (const char *in, gboolean *encoded)
 {
-	register const unsigned char *inptr;
-	unsigned char *outbuf = NULL;
+	register const unsigned char *inptr = (const unsigned char *) in;
+	const unsigned char *instart = inptr;
 	iconv_t cd = (iconv_t) -1;
 	const char *charset = NULL;
+	char *outbuf = NULL;
 	unsigned char c;
 	char *outstr;
 	GString *out;
 	
 	*encoded = FALSE;
 	
-	for (inptr = in; *inptr && inptr - in < GMIME_FOLD_LEN; inptr++)
+	while (*inptr && ((inptr - instart) < GMIME_FOLD_LEN)) {
 		if (*inptr > 127)
 			break;
+		inptr++;
+	}
 	
 	if (*inptr == '\0')
 		return g_strdup (in);
@@ -752,13 +762,13 @@ encode_param (const unsigned char *in, gboolean *encoded)
 		g_mime_iconv_close (cd);
 		if (outbuf == NULL) {
 			charset = "UTF-8";
-			inptr = in;
+			inptr = instart;
 		} else {
-			inptr = outbuf;
+			inptr = (const unsigned char *) outbuf;
 		}
 	} else {
 		charset = "UTF-8";
-		inptr = in;
+		inptr = instart;
 	}
 	
 	/* FIXME: set the 'language' as well, assuming we can get that info...? */
