@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 1999-2006 Jeffrey Stedfast
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 
@@ -120,7 +120,7 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
 	
 	g_return_val_if_fail (mem->buffer != NULL, -1);
 	
-	bound_end = stream->bound_end != -1 ? stream->bound_end : mem->buffer->len;
+	bound_end = stream->bound_end != -1 ? stream->bound_end : (off_t) mem->buffer->len;
 	
 	n = MIN (bound_end - stream->position, (off_t) len);
 	if (n > 0) {
@@ -190,7 +190,7 @@ stream_eos (GMimeStream *stream)
 	
 	g_return_val_if_fail (mem->buffer != NULL, TRUE);
 	
-	bound_end = stream->bound_end != -1 ? stream->bound_end : mem->buffer->len;
+	bound_end = stream->bound_end != -1 ? stream->bound_end : (off_t) mem->buffer->len;
 	
 	return stream->position >= bound_end;
 }
@@ -200,11 +200,7 @@ stream_reset (GMimeStream *stream)
 {
 	GMimeStreamMem *mem = (GMimeStreamMem *) stream;
 	
-	g_return_val_if_fail (mem->buffer != NULL, -1);
-	
-	stream->position = stream->bound_start;
-	
-	return 0;
+	return mem->buffer ? 0 : -1;
 }
 
 static off_t
@@ -215,7 +211,7 @@ stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence)
 	
 	g_return_val_if_fail (mem->buffer != NULL, -1);
 	
-	bound_end = stream->bound_end != -1 ? stream->bound_end : mem->buffer->len;
+	bound_end = stream->bound_end != -1 ? stream->bound_end : (off_t) mem->buffer->len;
 	
 	switch (whence) {
 	case GMIME_STREAM_SEEK_SET:
@@ -257,7 +253,7 @@ stream_length (GMimeStream *stream)
 	
 	g_return_val_if_fail (mem->buffer != NULL, -1);
 	
-	bound_end = stream->bound_end != -1 ? stream->bound_end : mem->buffer->len;
+	bound_end = stream->bound_end != -1 ? stream->bound_end : (off_t) mem->buffer->len;
 	
 	return bound_end - stream->bound_start;
 }
@@ -341,7 +337,7 @@ g_mime_stream_mem_new_with_buffer (const char *buffer, size_t len)
 	mem->owner = TRUE;
 	mem->buffer = g_byte_array_new ();
 	
-	g_byte_array_append (mem->buffer, buffer, len);
+	g_byte_array_append (mem->buffer, (unsigned char *) buffer, len);
 	
 	g_mime_stream_construct (GMIME_STREAM (mem), 0, -1);
 	
@@ -360,8 +356,6 @@ g_mime_stream_mem_new_with_buffer (const char *buffer, size_t len)
 GByteArray *
 g_mime_stream_mem_get_byte_array (GMimeStreamMem *mem)
 {
-	GMimeStream *stream;
-	
 	g_return_val_if_fail (GMIME_IS_STREAM_MEM (mem), NULL);
 	
 	return mem->buffer;
